class Animation
  constructor: ->
    @url = '/assets/animations'

    @name = null
    @frameCount = 0
    @keyframes = []
    @bonesIds = {}

  load: (name) ->
    self = this

    # Load the json animation file
    request = new XMLHttpRequest()
    request.open('GET', "#{@url}/#{name}.wglanim", true)

    request.onload = ->
      anim = JSON.parse(@responseText)
      self.parseAnim(anim)

    request.send(null)

  parseAnim: (anim) ->
    @name = anim.name
    @frameCount = anim.frameCount
    @keyframes = anim.keyframes

    # Build a table to lookup the bone id's
    for value, i in anim.bones
      @bonesIds[anim.bones[i]] = i

    # Force all bones to use efficient data structures
    for keyframe of @keyframes
      for bone of keyframe
        bone.pos = vec3.create(bone.pos)
        bone.rot = quat4.create(bone.rot)

  evaluate: (frameId, model, viewport, cursor) ->
    bones = model.bones
    return unless bones

    frameId = frameId % @frameCount
    frame = @keyframes[frameId]

    deg = cursor.y - (viewport.width / 2) / cursor.x - (viewport.height / 2)
    rad = deg * (Math.PI / 180)

    # Go in the order that the model specifies, will always process
    # parents first
    for bone in bones
      boneId = @bonesIds[bone.name]

      if boneId?
        frameBone = frame[boneId]
        bone.pos = frameBone.pos
        bone.rot = frameBone.rot

      # Rotate upper right arm
      if bone.name is 'arm_upper_r'
        mat = quat4.toMatrix(bone.rot)
        mat4.rotateX(mat, rad)
        quat = mat4.toQuaternion(mat)
        bone.rot = [quat[0], quat[1], quat[2], quat[3]]

      # No parent? No transform needed
      if bone.parent isnt -1
        parent = bones[bone.parent]

        # Apply the parent transform to this bone
        quat4.multiplyVec3(parent.worldRot, bone.pos, bone.worldPos)
        vec3.add(bone.worldPos, parent.worldPos)
        quat4.multiply(parent.worldRot, bone.rot, bone.worldRot)

      # We only need to compute the matrices for bones that actually have
      # vertices assigned to them
      if bone.skinned
        mat4.fromRotationTranslation(bone.worldRot, bone.worldPos, bone.boneMat)
        mat4.multiply(bone.boneMat, bone.bindPoseMat)

    # Notify the model that it needs to update it's bone matrices
    model.dirtyBones = true
