class Player
  constructor: ->
    @animations = {}
    @animation = 'hover'
    @direction = 'left'
    @state = {}

    @frameId = 0
    @angle = - Math.PI / 2

    @x = 0
    @y = 1
    @vx = 0 # Horizontal velocity
    @vy = 0 # Vertical velocity
    @ax = 0 # Movement force

  loadAnimations: (animations) ->
    for anim in animations
      @animations[anim] = new Animation()
      @animations[anim].load(anim)

  turn: (direction) ->
    return unless @y is -1
    @state[direction] = true

  idle: ->
    @state.idle = true
    @ax = 0

  run: ->
    @state.run = true
    return unless @y is -1
    @ax = 350
    @ax *= -1 if @direction is 'left'

  jump: ->
    @state.jump = true
    return unless @y is -1
    @vy = 60

  tranform: (model, delta, viewport, cursor) ->
    @position(delta)
    @rotate()
    @animate()

    @animations[@animation].evaluate(@frameId, model, viewport, cursor)

  position: (delta) ->
    t = delta * 0.0001
    g = 981 # Gravity
    bx = 550 # Braking force
    sp = 35 # Max speed

    @y += @vy * t
    @vy -= g * t

    @x += @vx * t

    if @vx > 0 and @ax <= 0
      bx = Math.abs(bx) * -1
    else if @vx < 0 and @ax >= 0
      bx = Math.abs(bx)
    else
       bx = 0

    if @y > -1
      bx = 0 if @ax is 0
      @vx = @vx + (@ax * t) + (bx * t)
    else
      if Math.abs(bx * t) > Math.abs(@vx) and @ax == 0
        @vx = 0
      else
        @vx = @vx + (@ax * t) + (bx * t)

    if @y <= -1
      @y = -1
      @vy = 0

    @vx = sp if @vx >= sp
    @vx = sp * -1 if @vx <= sp * -1

  rotate: ->
    if @state.left
      @angle -= 0.4
      if @angle <= -1.57
        @angle = -1.57
        @direction = 'left'
        delete @state.left
    else if @state.right
      @angle += 0.4
      if @angle >= 1.57
        @angle = 1.57
        @direction = 'right'
        delete @state.right

  animate: ->
    nextAnim = switch @animation
      when 'idle'
        if @state.jump
          'jump_idle'
        else if @state.run
          'start'
      when 'run'
        if @state.jump
          if @isAnimPassed(1)
            'jump_right'
          else if @isAnimPassed(13)
            'jump_left'
        else if @state.idle
          if @isAnimPassed(9)
            'stop_right'
          else if @isAnimPassed(21)
            'stop_left'
      when 'hover'
        break if @vy > 0 or @y > 0
        if @state.run then 'land_run' else 'land_idle'
      when 'start'
        'run' if @isAnimPassed()
      when 'jump_right', 'jump_left', 'jump_idle'
        'hover' if @isAnimPassed()
      when 'stop_right', 'stop_left'
        'idle' if @isAnimPassed()
      when 'land_run'
        'run' if @isAnimPassed()
      when 'land_idle'
        'idle' if @isAnimPassed()

    @startAnimation(nextAnim) if nextAnim

  startAnimation: (name) ->
    @state = {}
    @frameId = 0
    @animation = name

  isAnimPassed: (frame = null) ->
    frameCount = @animations[@animation].frameCount
    return (@frameId % frameCount is frame) if frame
    @frameId >= frameCount
