class Rendering
  constructor: (gl, @viewport, @input) ->
    gl.clearColor(0, 0, 0, 0)
    gl.clearDepth(1.0)
    gl.enable(gl.DEPTH_TEST)
    gl.enable(gl.CULL_FACE)
    gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true)

    @projectionMat = mat4.create()
    aspect = @viewport.width / @viewport.height
    mat4.perspective(45, aspect, 1.0, 4096.0, @projectionMat)

    @model = new Model()
    @model.load(gl, 'player', 'skinned')

    @weapon = new Model()
    @weapon.load(gl, 'pistol', 'static')

    animations = ['idle', 'start', 'stop_left', 'stop_right', 'run',
      'jump_idle', 'jump_left', 'jump_right', 'hover', 'land_idle', 'land_run']

    @player = new Player()
    @player.loadAnimations(animations)

    @lastFrame = new Date().getTime()

  draw: (gl, time) ->
    return unless @model.complete

    # Increase the animation frame
    delta = time - @lastFrame

    if delta > 17
      @lastFrame = time
      @player.frameId += 1

    # Update the animation
    if @input.keyPressed('left') or @input.keyPressed('right')
      @player.run()
    else
      @player.idle()

    if @input.keyPressed('up')
      @player.jump()

    if @player.direction isnt 'left' and @input.keyPressed('left') and !@input.keyPressed('right')
      @player.turn('left')
    else if @player.direction isnt 'right' and @input.keyPressed('right') and !@input.keyPressed('left')
      @player.turn('right')

    # Apply the tranforms of the given frame to the model
    @player.tranform(@model, delta, @viewport, @input.cursorPosition)

    # Set player position
    pViewMat = mat4.create()
    mat4.identity(pViewMat)
    mat4.translate(pViewMat, [@player.x, @player.y, -9])
    mat4.rotateY(pViewMat, @player.angle)

    # Set weapon position
    wViewMat = mat4.create()
    mat4.identity(wViewMat)
    mat4.translate(wViewMat, [@player.x, @player.y, -5])
    mat4.rotateY(wViewMat, -1.57)
    mat4.rotateX(wViewMat, -1.57)
    mat4.rotateZ(wViewMat, 1.57)
    mat4.rotateZ(wViewMat, @player.angle)

    # Draw models
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT)
    @model.draw(gl, pViewMat, @projectionMat)
    @weapon.draw(gl, wViewMat, @projectionMat)
